function gadget:GetInfo()
	return {
		name = "Buildings",
		desc = "Lets Buildings act as fake transports.",
		author = "Argh",
		date = "December 2nd, 2008",
		license = "(C) Wolfe Games, 2008",
		layer = 1,
		enabled = true,
	}
end

local math_rand = math.random
local GetUnitPosition = Spring.GetUnitPosition
local GiveOrderToUnit = Spring.GiveOrderToUnit
local x,y,z,randomX,randomZ,heading = 0,0,0,0,0,0
local piece1, piece2, piece3, piece4,door1 = 0,0,0,0,0
local GetUnitsInSphere = Spring.GetUnitsInSphere 
local GetUnitScriptPiece = Spring.GetUnitScriptPiece
local GetUnitPiecePosDir = Spring.GetUnitPiecePosDir
local CallCOBScript = Spring.CallCOBScript
local GetUnitDefID = Spring.GetUnitDefID
local CreateUnit = Spring.CreateUnit
local DestroyUnit = Spring.DestroyUnit
local GetUnitTeam = Spring.GetUnitTeam

local SetUnitNeutral = Spring.SetUnitNeutral
local Gaia = Spring.GetGaiaTeamID()

local unitDef
local Infantry = {}
local Holders = {}
local diesWithBuildings = {}

local CMD_BUILDING_GET = 32900
local CMD_BUILDING_RELEASE = 32901

local unitGroupNum = {}
local baseX, baseY, baseZ = 0,0,0
local fillNumber = 0
local fillAmount = 0
local fillList = {}
local piece1, door1, door, piece
local pieceX,pieceY,pieceZ = 0,0,0
local myTeam = 0
local amount = 0
local a,b,c = 0,0,0
local x,y,z = 0,0,0
local createString

local DeleteList = {}
local ToggleList = {}

if (gadgetHandler:IsSyncedCode()) then
--////////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////////
function gadget:Initialize()
	for ud,_ in pairs(UnitDefs) do
		if UnitDefs[ud].customParams.unit_ui_category == 'infantry' then
			table.insert(Infantry,ud,1)
		end
		if UnitDefs[ud].customParams.can_be_captured == 'yes' then
			table.insert(Holders,ud,tonumber(UnitDefs[ud].customParams.hold_amount))
		end
		if UnitDefs[ud].customParams.dies_with_buildings == 'yes' then
			table.insert(diesWithBuildings,ud,1)
		end
	end
end

BuildingGet = {
		id=CMD_BUILDING_GET,
		type=CMDTYPE.ICON,
		name="",
		onlyTexture=true,
		texture="&.9x.9&bitmaps/icons/blank.tif&bitmaps/icons/BUILDING_ENTER.png",
		tooltip = "Brings nearby infantry inside this building.",
		}

BuildingRelease = {
		id=CMD_BUILDING_RELEASE,
		type=CMDTYPE.ICON,
		name="",
		onlyTexture=true,
		texture="&.9x.9&bitmaps/icons/blank.tif&bitmaps/icons/BUILDING_EXIT.png",
		tooltip = "Releases infantry from this building.",
		}

function gadget:UnitCreated(u,ud,team)
	if Holders[ud] then
		Spring.InsertUnitCmdDesc(u, BuildingGet)
		Spring.InsertUnitCmdDesc(u, BuildingRelease)
	end
end

function gadget:UnitDestroyed(u,ud,team)
	if Holders[ud] then
		CallCOBScript(u, "ReturnPieces",0)
		piece = GetUnitScriptPiece(u,piece1)

		x,y,z  = GetUnitPiecePosDir(u,piece)
		unitGroupNum = GetUnitsInSphere(x, y, z,32)
		for _,me in ipairs (unitGroupNum) do
			if me ~= u and diesWithBuildings[GetUnitDefID(me)] then
				DelayedCall(function()
					SetUnitNeutral(me,true)
					DestroyUnit(me,true,false)
				end)
			end
		end
	end
end

function gadget:AllowCommand(u, ud, team, cmd, param, opts, tag, sync)
	if cmd == CMD.GUARD then
		if Infantry[ud] and DeleteList[u] == nil then
			table.insert(DeleteList, u, 1)
			if Holders[GetUnitDefID(param[1])] then
				GiveOrderToUnit(param[1],CMD_BUILDING_GET,{0},0)
				return true
			else
				return true
			end
		else
			return true
		end
	end
	return true
end

function PieceNums(u,ud,team,p1)
	piece1 = p1
end
gadgetHandler:RegisterGlobal("PieceNums", PieceNums)

function DoorNums(u,ud,team,p1)
	door1 = p1
end
gadgetHandler:RegisterGlobal("DoorNums", DoorNums)

function gadget:UnitGiven(unitID, unitDefID, unitTeam, oldTeam)
	if Holders[unitDefID] then
		GiveOrderToUnit(unitID,CMD_BUILDING_GET,{0},0)
	end
end

--////////////////////////////////////////////////////////////////////  SAFE DELAYED FUNCTION
local delayedCalls = {}
function DelayedCall(fun)
	delayedCalls[#delayedCalls+1] = fun 
end


function gadget:GameFrame(f)
	if f % 32 < 0.1 then
		DeleteList = {}
		ToggleList = {}
	end
	if delayedCalls then
		for i=1,#delayedCalls do
			local fun = delayedCalls[i]
			fun()
		end
		delayedCalls = {}
	end
end
--////////////////////////////////////////////////////////////////////


function gadget:CommandFallback(u, ud, team, cmd, param, opts)
	if cmd  == CMD_BUILDING_GET then
	if Holders[ud] and ToggleList[u] == nil then
		table.insert(ToggleList,u,1)
		CallCOBScript(u, "ReturnPieces",0)
		piece = GetUnitScriptPiece(u,piece1)
		fillAmount = 0
		fillNumber = 0

		x,y,z = GetUnitPiecePosDir(u,piece)
		---Spring.Echo(x,y,z)
		fillList = GetUnitsInSphere(x,y,z,16)
		
		if #fillList == nil then fillNumber = 0 else fillNumber = #fillList end
		fillAmount = fillNumber

		
		--if fillAmount == 0 then Spring.Echo("No nearby infantry, move them closer to this building.") end

		amount = tonumber(UnitDefs[ud].customParams.hold_amount)
		--Spring.Echo(fillAmount,amount)
		unitGroupNum = nil
		unitGroupNum = GetUnitsInSphere(x,y,z,400)
		local numGuys = #unitGroupNum
		--Spring.Echo(numGuys)

		for k,me in ipairs (unitGroupNum) do
			--Spring.Echo(k,me)
			myTeam = GetUnitTeam(me)
			if me ~= u and Infantry[GetUnitDefID(me)] and team == myTeam or team == Gaia then
				--Spring.Echo("Found Infantry")
				--Spring.Echo(fillAmount,amount)
				--if fillAmount == 0 then fillAmount = -1 end
				if (amount > fillAmount) then
					fillAmount = fillAmount + 1
					if DeleteList[me] == nil or false then
						table.insert(DeleteList, me, 1)
						DelayedCall(function()
							unitDef = GetUnitDefID(me)			
							SetUnitNeutral(me,true)
							DestroyUnit(me,false,true)
							createString = tostring(UnitDefs[unitDef].customParams.spawn_in_building)
							you = CreateUnit(createString,x,y,z,0,team)	
							Spring.MoveCtrl.Enable(you)
							Spring.MoveCtrl.SetGravity(you,0)
							Spring.MoveCtrl.SetSlopeStop(you,false)
							Spring.MoveCtrl.SetCollideStop(you,false)
							Spring.MoveCtrl.SetTrackGround(you,false)
							Spring.MoveCtrl.SetPosition(you, x, y, z)
						end)
					end
					
				else
					Spring.SendMessageToTeam(team,"This building cannot hold any more infantry.")
					--Spring.Echo(amount,fillAmount)
					return true, true;
				end
			end
		end
	return true, true;
	end
	end
	if cmd  == CMD_BUILDING_RELEASE then
	if Holders[ud] and ToggleList[u] == nil then
		table.insert(ToggleList,u,1)
		CallCOBScript(u, "ReturnPieces",0)
		piece = GetUnitScriptPiece(u,piece1)

		CallCOBScript(u, "GetDoor",0)
		door = GetUnitScriptPiece(u,door1)
		pieceX, pieceY, pieceZ = GetUnitPiecePosDir(u,door)

		x,y,z  = GetUnitPiecePosDir(u,piece)
		unitGroupNum = GetUnitsInSphere(x, y, z,16)
		if unitGroupNum ~= nil then
		for _,me in ipairs (unitGroupNum) do
			myTeam = GetUnitTeam(me)
			if me ~= u and diesWithBuildings[GetUnitDefID(me)] and team == myTeam then
				if DeleteList[me] == nil then
					table.insert(DeleteList, me, 1)
					DelayedCall(function()
						unitDef = GetUnitDefID(me)
						createString = tostring(UnitDefs[unitDef].customParams.spawn_from_building)
						SetUnitNeutral(me,true)
						DestroyUnit(me,false,true)

						randomX = math_rand(-16,16)
						randomZ = math_rand(-16,16)
						you = CreateUnit(createString,pieceX - randomX, pieceY, pieceZ - randomZ,0,team)
						a,b,c = GetUnitPosition(you)
						GiveOrderToUnit(you,CMD.MOVE,{a + randomX, b, c + randomZ}, 0)
					end)
				end
			end
		end
		end
	return true, true;
	end
	end
	return false
end

end