function gadget:GetInfo()
	return {
		name = "COB_Gadgets",
		desc = "Standard Gadgets that communicate with COB, for P.U.R.E.",
		author = "Argh",
		date = "March 13th, 2008",
		license = "Copyright (C) Wolfe Games, 2008",
		layer = 1,
		enabled = true,
	}
end

local resistanceTrooper = {}
local DeleteList = {}

local CreateUnitList = {}

local SetUnitNeutral = Spring.SetUnitNeutral

if (gadgetHandler:IsSyncedCode()) then
--////////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////////
	function gadget:Initialize()
		NewGroup = 0
		UnitGroups = {GroupNumber, Individual = {},}
		gadgetHandler:RegisterGlobal("NewGroup", NewGroup)
		gadgetHandler:RegisterGlobal("UnitGroups", UnitGroups)
	end

--////////////////////////////////////////////////////////////////////
local function copyCommandQueue(targetID, unitID)

  local states = Spring.GetUnitStates(targetID)

  if (states ~= nil) then
    Spring.GiveOrderToUnit(unitID, CMD.FIRE_STATE, { states.firestate }, 0)
    Spring.GiveOrderToUnit(unitID, CMD.MOVE_STATE, { states.movestate }, 0)
    Spring.GiveOrderToUnit(unitID, CMD.REPEAT,     { states['repeat']  and 1 or 0 }, 0)
    Spring.GiveOrderToUnit(unitID, CMD.ONOFF,      { states.active     and 1 or 0 }, 0)
    Spring.GiveOrderToUnit(unitID, CMD.CLOAK,      { states.cloak      and 1 or 0 }, 0)
    Spring.GiveOrderToUnit(unitID, CMD.TRAJECTORY, { states.trajectory and 1 or 0 }, 0)
  end
 
  local queue = Spring.GetCommandQueue(targetID);
 
  if (queue ~= nil) then

    for k,v in ipairs(queue) do  --  in order
      local opts = v.options
      if (not opts.internal) then
        local newopts = {}
        if (opts.alt)   then table.insert(newopts, "alt")   end
        if (opts.ctrl)  then table.insert(newopts, "ctrl")  end
        if (opts.right) then table.insert(newopts, "right") end
        table.insert(newopts, "shift")
        Spring.GiveOrderToUnit(unitID, v.id, v.params, opts.coded)
      end
    end
  end

--COMMENTED OUT, NOT NEEDED, NO 1:1 REPLACEMENT / CLONE OF BUILDERS WITH ORDERS AT THIS TIME - GLW
  --[[local buildqueue = Spring.GetRealBuildQueue(targetID)
  if (buildqueue ~= nil) then
    for udid,buildPair in ipairs(buildqueue) do
      local udid, count = next(buildPair, nil)
      Spring.AddBuildOrders(unitID, udid, count)
    end
  end]]
 
end

--////////////////////////////////////////////////////////////////////
function BuilderThreePack(u, ud, team)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitBuildFacing(u)
	local one
	local two

	if heading == 0 or heading == 2 then
		DelayedCall(function()
			Spring.CreateUnit("Builder", x - 48,y,z, heading, team) 
			Spring.CreateUnit("Builder", x + 48,y,z, heading, team)
		end)
	end

	if heading == 1 or heading == 3 then
		DelayedCall(function()
			Spring.CreateUnit("Builder", x,y,z + 48, heading, team) 
			Spring.CreateUnit("Builder", x,y,z - 48, heading, team)
		end)
	end
end

--////////////////////////////////////////////////////////////////////
function BuildEngineeringTeamThreePack(u, ud, team)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitBuildFacing(u)
	local one
	local two

	if heading == 0 or heading == 2 then
		DelayedCall(function()
			Spring.CreateUnit("EngineeringTeam", x - 48,y,z, heading, team) 
			Spring.CreateUnit("EngineeringTeam", x + 48,y,z, heading, team)
		end)
	end

	if heading == 1 or heading == 3 then
		DelayedCall(function()
			Spring.CreateUnit("EngineeringTeam", x,y,z + 48, heading, team) 
			Spring.CreateUnit("EngineeringTeam", x,y,z - 48, heading, team)
		end)
	end
end

--////////////////////////////////////////////////////////////////////
function BuildResistance_HeavyTrooper5Pack(u, ud, team, factory)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitBuildFacing(u)
	local OrderQueue = {}
	local NewUnit
	
	if heading == 0 or heading == 2 then
	DelayedCall(function()

		NewUnit = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn", x - 24,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn", x + 24,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn", x - 16,y,z - 24, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn", x + 16,y,z - 24, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1

		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end
	end)
	end

	if heading == 1 or heading == 3 then
	DelayedCall(function()

		NewUnit = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn", x - 8 ,y,z + 24, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)	
		NewUnit = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn", x - 8,y,z - 24, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn", x - 24 ,y,z + 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn", x - 24,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1

		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end
	end)
	end



end

--////////////////////////////////////////////////////////////////////
function BuildLightMechThreePack(u, ud, team, factory)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitBuildFacing(u)
	local OrderQueue = {}
	local NewUnit

	if heading == 0 or heading == 2 then
	DelayedCall(function()

		NewUnit = Spring.CreateUnit("LightMech_Rocket", x - 24,y,z-16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("LightMech_AA", x + 24,y,z-16, heading, team)
        		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1

		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end
	end)
	end

	if heading == 1 or heading == 3 then
	DelayedCall(function()

		NewUnit = Spring.CreateUnit("LightMech_Rocket", x-16,y,z + 24, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("LightMech_AA", x-16,y,z - 24, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1

		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end
	end)
	end



end

--////////////////////////////////////////////////////////////////////
function BuildSniperFivePack(u, ud, team, factory)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitBuildFacing(u)
	local OrderQueue = {}
	local NewUnit

	if heading == 0 or heading == 2 then
	DelayedCall(function()

		NewUnit = Spring.CreateUnit("Resistance_Sniper", x - 8,y,z-8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Sniper", x - 16,y,z-6, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Sniper", x + 8,y,z-8, heading, team)
        		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Sniper", x + 16,y,z-16, heading, team)
        		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1

		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end

	end)
	end

	if heading == 1 or heading == 3 then
	DelayedCall(function()

		NewUnit = Spring.CreateUnit("Resistance_Sniper", x-8,y,z + 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Sniper", x-16,y,z + 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Sniper", x-8,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Sniper", x-16,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1

		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end

	end)
	end



end

--////////////////////////////////////////////////////////////////////
function BuildGrenadierFivePack(u, ud, team, factory)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitBuildFacing(u)
	local OrderQueue = {}
	local NewUnit

	if heading == 0 or heading == 2 then
		
	DelayedCall(function()

		NewUnit = Spring.CreateUnit("Resistance_Grenadier", x - 8,y,z-8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Grenadier", x - 16,y,z-6, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Grenadier", x + 8,y,z-8, heading, team)
        		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Grenadier", x + 16,y,z-16, heading, team)
        		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1

		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end
	
	end)
	end

	if heading == 1 or heading == 3 then
	DelayedCall(function()

		NewUnit = Spring.CreateUnit("Resistance_Grenadier", x-8,y,z + 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Grenadier", x-16,y,z + 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Grenadier", x-8,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("Resistance_Grenadier", x-16,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1

		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end

	end)
	end



end

--////////////////////////////////////////////////////////////////////

function BuildResistanceTrooperSquad(u,ud,team,factory)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitBuildFacing(u)
	local OrderQueue = {}
	local NewUnit

	if heading == 0 or heading == 2 then
	
	DelayedCall(function()
		--------------------------------------------------------- FRONT ROW
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x - 8,y,z, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x + 8,y,z, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x - 16,y,z, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x + 16,y,z, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		--------------------------------------------------------- MID ROW
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x - 8,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x + 8,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x - 16,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x + 16,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		--------------------------------------------------------- BACK ROW
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x - 8,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x + 8,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x - 16,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x + 16,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1
	
		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end

	end)			
	end

	if heading == 1 or heading == 3 then
	DelayedCall(function()
		--------------------------------------------------------- FRONT ROW
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x,y,z + 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x,y,z + 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		--------------------------------------------------------- MID ROW
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x - 8,y,z, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x - 8,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x - 8,y,z + 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x - 8,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Spawn", x  - 8,y,z + 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		--------------------------------------------------------- BACK ROW
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x - 16,y,z, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x - 16,y,z - 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x - 16,y,z + 8, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x - 16,y,z - 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)
		NewUnit = Spring.CreateUnit("ResistanceTrooper_Rocket", x - 16,y,z + 16, heading, team)
		table.insert(OrderQueue,{unit = NewUnit})
		copyCommandQueue(factory, NewUnit)

		--Puts the originating Unit into the Queue
		table.insert(OrderQueue,{unit = u})
		--Puts all members of OrderQueue into a Group, as Individuals
		NewGroup = NewGroup + 1
	
		for _,t in ipairs (OrderQueue) do
		local MyNewUnit = t.unit
		table.insert(UnitGroups,{GroupNumber = NewGroup, Individual = MyNewUnit})
		end

	end)
	end
end


--////////////////////////////////////////////////////////////////////

function TransformMeAdvanceBaseMobile(u, ud, team)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitBuildFacing(u)

	Spring.SetUnitNoDraw(u,true)
	Spring.SetUnitNoSelect(u,true)
	Spring.SetUnitNoMinimap(u,true)
	local health = Spring.GetUnitHealth(u)

	SetUnitNeutral(u,true)

	DelayedCall(function()
		Spring.DestroyUnit(u,false,true)
		if DeleteList[u] == nil then
			table.insert(DeleteList, u, 1)
			local y = Spring.CreateUnit("AdvanceBase_Mobile", x,y,z, heading, team)
			Spring.SetUnitHealth(y,health)
			SendToUnsynced("SelectMe", y, team)
		end
	end)
end
gadgetHandler:RegisterGlobal("TransformMeAdvanceBaseMobile", TransformMeAdvanceBaseMobile)

--////////////////////////////////////////////////////////////////////

function TransformMeAdvanceBaseStatic(u, ud, team)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitHeading(u)
	
	--Spring.Echo (heading)
	
	if heading < 1600 and heading > -1600 then heading = 0 end
	if heading < 17000 and heading > 15000 then heading = 1 end
	if heading > 31000 and heading < 32769 or heading < -31000 and heading > -32769 then heading = 2 end
	if heading < -15000 and heading > -17000 then heading = 3 end

	Spring.SetUnitNoDraw(u,true)
	Spring.SetUnitNoSelect(u,true)
	Spring.SetUnitNoMinimap(u,true)
	local health = Spring.GetUnitHealth(u)

	SetUnitNeutral(u,true)

	DelayedCall(function()
		Spring.DestroyUnit(u,false,true)

		if DeleteList[u] == nil then
			table.insert(DeleteList, u, 1)
			local y = Spring.CreateUnit("AdvanceBase", x,y,z, heading, team)
			Spring.SetUnitHealth(y,health)
			SendToUnsynced("SelectMe", y, team)
		end
	end)
end
gadgetHandler:RegisterGlobal("TransformMeAdvanceBaseStatic", TransformMeAdvanceBaseStatic)

--////////////////////////////////////////////////////////////////////

function TransformMeAirBaseMobile(u, ud, team)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitBuildFacing(u)

	Spring.SetUnitNoDraw(u,true)
	Spring.SetUnitNoSelect(u,true)
	Spring.SetUnitNoMinimap(u,true)
	local health = Spring.GetUnitHealth(u)

	SetUnitNeutral(u,true)

	DelayedCall(function()
		Spring.DestroyUnit(u,false,true)

		if DeleteList[u] == nil then
			table.insert(DeleteList, u, 1)
			local y = Spring.CreateUnit("AirBase_Mobile", x,y,z, heading, team)
			Spring.SetUnitHealth(y,health)
			SendToUnsynced("SelectMe", y, team)
		end
	end)
end
gadgetHandler:RegisterGlobal("TransformMeAirBaseMobile", TransformMeAirBaseMobile)

--////////////////////////////////////////////////////////////////////

function TransformMeAirBaseStatic(u, ud, team)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitHeading(u)
	
	if heading < 1600 and heading > -1600 then heading = 0 end
	if heading < 17000 and heading > 15000 then heading = 1 end
	if heading > 31000 and heading < 32769 or heading < -31000 and heading > -32769 then heading = 2 end
	if heading < -15000 and heading > -17000 then heading = 3 end

	Spring.SetUnitNoDraw(u,true)
	Spring.SetUnitNoSelect(u,true)
	Spring.SetUnitNoMinimap(u,true)
	local health = Spring.GetUnitHealth(u)

	SetUnitNeutral(u,true)

	DelayedCall(function()
		Spring.DestroyUnit(u,false,true)

		if DeleteList[u] == nil then
			table.insert(DeleteList, u, 1)
			local y = Spring.CreateUnit("AirBase", x,y,z, heading, team)
			Spring.SetUnitHealth(y,health)
			SendToUnsynced("SelectMe", y, team)
		end
	end)
end
gadgetHandler:RegisterGlobal("TransformMeAirBaseStatic", TransformMeAirBaseStatic)

--////////////////////////////////////////////////////////////////////

function TransformMeResistanceHeavyTrooperSpawnAirMode(u, ud, team)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitHeading(u)
	
	--Spring.Echo (heading)
	
	if heading < 1600 and heading > -1600 then heading = 0 end
	if heading < 17000 and heading > 15000 then heading = 1 end
	if heading > 31000 and heading < 32769 or heading < -31000 and heading > -32769 then heading = 2 end
	if heading < -15000 and heading > -17000 then heading = 3 end

	Spring.SetUnitNoDraw(u,true)
	Spring.SetUnitNoSelect(u,true)
	Spring.SetUnitNoMinimap(u,true)
	local health = Spring.GetUnitHealth(u)

	SetUnitNeutral(u,true)

	DelayedCall(function()
		Spring.DestroyUnit(u,false,true)

		if DeleteList[u] == nil then
			table.insert(DeleteList, u, 1)
			local NewGuy = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn_Airmode", x,y,z, heading, team)
			copyCommandQueue(u, NewGuy)

			for k,v in ipairs (UnitGroups) do
				if u == v.Individual then
					table.insert(UnitGroups,{GroupNumber = v.GroupNumber, Individual = NewGuy})
				end
			end	
			Spring.SetUnitHealth(NewGuy,health)
			SendToUnsynced("SelectMe", NewGuy, team)
		end
	end)
end
gadgetHandler:RegisterGlobal("TransformMeResistanceHeavyTrooperSpawnAirMode", TransformMeResistanceHeavyTrooperSpawnAirMode)

--////////////////////////////////////////////////////////////////////

function TransformMeResistanceHeavyTrooperSpawnGroundMode(u, ud, team)

	local x, y, z = Spring.GetUnitPosition(u)
	local heading = Spring.GetUnitHeading(u)
	
	--Spring.Echo (heading)

	if heading < 8192 and heading > -8192 then heading = 0 end
	if heading < 24576 and heading > 8193 then heading = 1 end
	if heading > 24576 and heading < 32769 or heading < -24576 and heading > -32769 then heading = 2 end
	if heading < -24576 and heading > -8193 then heading = 3 end

	-- heading is not always returned properly by aircraft
	if heading ~= 0 and heading ~= 1 and heading ~= 2 and heading ~= 3 then heading = 0 end

    	Spring.SetUnitNoDraw(u,true)
    	Spring.SetUnitNoSelect(u,true)
    	Spring.SetUnitNoMinimap(u,true)
    	local health = Spring.GetUnitHealth(u)
	SetUnitNeutral(u,true)

	DelayedCall(function()
    		Spring.DestroyUnit(u,false,true)	

		if DeleteList[u] == nil then
			table.insert(DeleteList, u, 1)
			local NewGuy = Spring.CreateUnit("Resistance_HeavyTrooper_Spawn", x,y,z, heading, team)
			copyCommandQueue(u, NewGuy)
			for k,v in ipairs (UnitGroups) do
				if u == v.Individual then
					table.insert(UnitGroups,{GroupNumber = v.GroupNumber, Individual = NewGuy})
				end
			end
			Spring.SetUnitHealth(NewGuy,health)
			SendToUnsynced("SelectMe", NewGuy, team)
		end
	end)
end
gadgetHandler:RegisterGlobal("TransformMeResistanceHeavyTrooperSpawnGroundMode", TransformMeResistanceHeavyTrooperSpawnGroundMode)

--////////////////////////////////////////////////////////////////////  SAFE DELAYED COMMANDS
local delayedCalls = {}
function DelayedCall(fun)
	delayedCalls[#delayedCalls+1] = fun 
end

function gadget:GameFrame(f)
	if f % 32 < 0.1 then
		DeleteList = {}
	end

	for i=1,#delayedCalls do
		local fun = delayedCalls[i]
		fun()
	end
	delayedCalls = {}
end
--////////////////////////////////////////////////////////////////////

function RemoveMe(u, ud, team)
	DelayedCall(function()
		Spring.SetUnitNoDraw(u,true)
		Spring.SetUnitNoSelect(u,true)
		Spring.SetUnitNoMinimap(u,true)
		Spring.DestroyUnit(u,false,true)
	end)
end
gadgetHandler:RegisterGlobal("RemoveMe", RemoveMe)
--////////////////////////////////////////////////////////////////////

function CheckState(u, ud, team, success)
	if Spring.GetUnitIsBuilding(u) == nil then 
		success = 0 
		return u, ud, team, success
	else
		success = 1
		return u, ud, team, success
	end
end
gadgetHandler:RegisterGlobal("CheckState", CheckState)

function gadget:UnitFinished(u, ud, team)
	local f = Spring.GetGameFrame()
	if f < 10 then
		if GG.MissionRunning ~= 1 then
			local unitDefID = Spring.GetUnitDefID(u)
			if UnitDefs[unitDefID].name == "overmindcommander" then
				BuilderThreePack(u,ud,team)
			end
			if UnitDefs[unitDefID].name == "megaconstructor" then
				BuildEngineeringTeamThreePack(u,ud,team)
			end
		end
	end
end

function gadget:UnitFromFactory(unitID, unitDefID, unitTeam, factID, factDefID, userOrders)
	if UnitDefs[unitDefID].name == "resistancetrooper" then
		BuildResistanceTrooperSquad(unitID,unitDefID,unitTeam,factID)
	end

	if UnitDefs[unitDefID].name == "lightmech" then
		BuildLightMechThreePack(unitID,unitDefID,unitTeam,factID)
	end

	if UnitDefs[unitDefID].name == "resistance_sniper" then
		BuildSniperFivePack(unitID,unitDefID,unitTeam,factID)
	end

	if UnitDefs[unitDefID].name == "resistance_grenadier" then
		BuildGrenadierFivePack(unitID,unitDefID,unitTeam,factID)
	end

	if UnitDefs[unitDefID].name == "resistance_heavytrooper" then
		BuildResistance_HeavyTrooper5Pack(unitID,unitDefID,unitTeam,factID)
	end
end
--////////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////
else
--////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////////UNSYNC
local TeamID = Spring.GetLocalTeamID()

function gadget:RecvFromSynced(name,unitID,teamValue)
	if (name == 'SelectMe') and TeamID == teamValue then		
		Spring.SelectUnitArray({unitID},true)
  	end
	return false
end
--////////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////////
--////////////////////////////////////////////////////////////////////
end	