function widget:GetInfo()
	return {
		name = "P.U.R.E. Team Display (SELECTED)",
		desc = "Shows a team color circle or direction indicator for units, if selected",
		author = "Argh",
		date = "August 21, 2008",
		license = "(C) Wolfe Games 2008",
		layer = -2,
		enabled = false
		}
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
local BuilderList = {}

local LandList = {}									

local AirCircleList = {}

local SoldierList = {}

local StaticList = {}

local StaticBuilderList = {}

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--
-- Variable Declarations
--
local glResetState = gl.ResetState
local GL_SRC_ALPHA = GL.SRC_ALPHA
local GL_ONE = GL.ONE
local glTexCoord = gl.TexCoord
local glVertex = gl.Vertex
local glCreateList = gl.CreateList
local glBeginEnd = gl.BeginEnd
local GL_QUADS = GL.QUADS
local GetVisibleUnits = Spring.GetVisibleUnits
local GetUnitTeam = Spring.GetUnitTeam
local GetUnitRadius = Spring.GetUnitRadius
local GetUnitBasePosition = Spring.GetUnitBasePosition
local GetUnitHeading = Spring.GetUnitHeading
local GetUnitDefID = Spring.GetUnitDefID
local IsUnitSelected = Spring.IsUnitSelected
local GetTeamColor = Spring.GetTeamColor
local IsUnitVisible = Spring.IsUnitVisible
local GetGroundNormal = Spring.GetGroundNormal
local glLineWidth = gl.LineWidth
local glDepthTest = gl.DepthTest
local glPolygonOffset = gl.PolygonOffset
local glColor = gl.Color
local glTexture = gl.Texture
local GL_LEQUAL = GL.LEQUAL
local glBlending = gl.Blending
local glDepthMask = gl.DepthMask
local glDrawListAtUnit = gl.DrawListAtUnit
local UnitIDList, id, teamID, r, g, b
local radius, direction, directionTwo
local texture =   "LuaUI/Graphics/ArrowHead.tga"
local textureTwo =   "LuaUI/Graphics/AlphaBlob.tga"
local textureThree =   "LuaUI/Graphics/SoldierBlob.tga"
local textureFour =   "LuaUI/Graphics/StaticBlob.tga"
local textureFive =   "LuaUI/Graphics/BuildDistance.tga"
local turnMe = 0
local file

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function widget:Initialize()

	Spring.SendCommands("luaui disablewidget P.U.R.E. Team Display (ALL)")

	for ud,_ in pairs(UnitDefs) do
		if UnitDefs[ud].customParams.team_circle_param == 'builder' then
			table.insert(BuilderList,ud,1)
		end
		if UnitDefs[ud].customParams.team_circle_param == 'land' then
			table.insert(LandList,ud,1)
		end
		if UnitDefs[ud].customParams.team_circle_param == 'air' then
			table.insert(AirCircleList,ud,1)
		end
		if UnitDefs[ud].customParams.team_circle_param == 'soldier' then
			table.insert(SoldierList,ud,1)
		end
		if UnitDefs[ud].customParams.team_circle_param == 'static' then
			table.insert(StaticList,ud,1)
		end
		if UnitDefs[ud].customParams.team_circle_param == 'static_builder' then
			table.insert(StaticBuilderList,ud,1)
		end
	end	

	SimpleCircle = glCreateList(function()
		glBeginEnd(GL_QUADS, function()
			glTexCoord(0.04,0.04)
			glVertex(-1,0,-1)
			glTexCoord(0.96,0.04)
			glVertex(1,0,-1)
			glTexCoord(0.96,0.96)
			glVertex(1,0,1)
			glTexCoord(0.04,0.96)
			glVertex(-1,0,1)
		end)
	end)
	arrowHead = glCreateList(function()
		glBeginEnd(GL_QUADS, function()
			glTexCoord(0.04,0.04)
			glVertex(-1,0,-0.8)
			glTexCoord(0.96,0.04)
			glVertex(1,0,-0.8)
			glTexCoord(0.96,0.96)
			glVertex(1,0,1.2)
			glTexCoord(0.04,0.96)
			glVertex(-1,0,1.2)
		end)
	end)

	file = io.open('newfile.tmp', 'w+')
  	if (file) then
		file:write('unitBox  0 0 0 0')
		file:close()
		Spring.SendCommands({'cmdcolors newfile.tmp'})
  	end
end


function widget:GameFrame(f)
	if f > 0 then
		turnMe = turnMe -0.25
		if turnMe < 0 then turnMe = 360 end
	end
end

function widget:Shutdown()
	gl.DeleteList(SimpleCircle)
	gl.DeleteList(ArrowHead)
	Spring.SendCommands({'cmdcolors cmdcolors.txt'})
end
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------



function widget:DrawWorldPreUnit()


	unitIDList = GetVisibleUnits(-1,3000,false)

	if unitIDList[1] ~= nil then
	
  		glBlending(GL_SRC_ALPHA, GL_ONE)
  		glDepthTest(true)
		glDepthMask(false)
	
		for _,unitID in ipairs(unitIDList) do
			if (IsUnitVisible(unitID)) then
				id = GetUnitDefID(unitID)
				teamID = GetUnitTeam(unitID)
				if (teamID) then
					if BuilderList[id] then
						if IsUnitSelected(unitID) then
							direction = GetUnitHeading(unitID)
							if direction < 0 then directionTwo = direction * -1 else directionTwo = direction end
							radius = 6*UnitDefs[id].xsize
							glTexture(texture)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,1.0)
							glDrawListAtUnit(unitID, arrowHead, false, radius, 1.0, radius, directionTwo/182, 0, direction, 0)

							radius = UnitDefs[id].buildDistance
							glTexture(textureFive)
							glColor(1.0,0.7,0,0.5)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, turnMe, 0, 1.0, 0)
						else
							direction = GetUnitHeading(unitID)
							if direction < 0 then directionTwo = direction * -1 else directionTwo = direction end
							radius = 6*UnitDefs[id].xsize
							glTexture(texture)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,0.3)
							glDrawListAtUnit(unitID, arrowHead, false, radius, 1.0, radius, directionTwo/182, 0, direction, 0)
						end
					end
					if LandList[id] then
						if IsUnitSelected(unitID) then
							radius = 6*UnitDefs[id].xsize
							glTexture(texture)
	 						direction = GetUnitHeading(unitID)
							if direction < 0 then directionTwo = direction * -1 else directionTwo = direction end
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,1.0)
							glDrawListAtUnit(unitID, arrowHead, false, radius, 1.0, radius, directionTwo/182, 0, direction, 0)
						else
							radius = 6*UnitDefs[id].xsize
							glTexture(texture)
	 						direction = GetUnitHeading(unitID)
							if direction < 0 then directionTwo = direction * -1 else directionTwo = direction end
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,0.3)
							glDrawListAtUnit(unitID, arrowHead, false, radius, 1.0, radius, directionTwo/182, 0, direction, 0)
						end
					end
					if AirCircleList[id] then
						if IsUnitSelected(unitID) then
							radius = 6*UnitDefs[id].xsize
							glTexture(textureTwo)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,1.0)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, 0, 0, 0, 0)
						else
							radius = 6*UnitDefs[id].xsize
							glTexture(textureTwo)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,0.3)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, 0, 0, 0, 0)
						end
					end
					if SoldierList[id] then
						if IsUnitSelected(unitID) then
							radius = 6*UnitDefs[id].xsize
							glTexture(textureThree)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,1.0)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, 0, 0, 0, 0)
						else
							radius = 6*UnitDefs[id].xsize
							glTexture(textureThree)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,0.3)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, 0, 0, 0, 0)
						end
					end
					if StaticList[id] then
						if IsUnitSelected(unitID) then
							radius = 6*UnitDefs[id].xsize
							glTexture(textureFour)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,1.0)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, 0, 0, 0, 0)
						else
							radius = 6*UnitDefs[id].xsize
							glTexture(textureFour)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,0.3)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, 0, 0, 0, 0)
						end
					end
					if StaticBuilderList[id] then
						if IsUnitSelected(unitID) then
							radius = 6*UnitDefs[id].xsize
							glTexture(textureFour)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,1.0)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, 0, 0, 0, 0)

							radius = UnitDefs[id].buildDistance
							glTexture(textureFive)
							glColor(1.0,0.7,0,0.5)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, turnMe, 0, 1.0, 0)
						else
							radius = 6*UnitDefs[id].xsize
							glTexture(textureFour)
							r,g,b = GetTeamColor(teamID)
							glColor(r,g,b,0.3)
							glDrawListAtUnit(unitID, SimpleCircle, false, radius, 1.0, radius, 0, 0, 0, 0)
						end
					end
				end
			end
		end
  		glResetState()
	end
	
	
end
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------